package ch.capi.geom 
{
	import ch.capi.display.IDrawable;	
	import ch.capi.utils.ICloneable;	
	import ch.capi.display.IDragDelimiter;	
	
	import flash.display.Graphics;	
	import flash.geom.Point;	
	
	/**
	 * Represents a circle.
	 * 
	 * @author		Cedric Tabin - thecaptain
	 * @version 	1.0
	 */
	public class BasicCircle implements IDragDelimiter, ICloneable, IDrawable
	{
		//---------//
		//Variables//
		//---------//
		private var _radius:Number;
		
		/**
		 * The x coordinate of the center.
		 */
		public var x:Number;
		
		/**
		 * The y coordinate of the center.
		 */
		public var y:Number;
		
		
		//-----------------//
		//Getters & Setters//
		//-----------------//
		
		/**
		 * The radius of the <code>BasicCircle</code>. This value must not be negative.
		 */
		public function get radius():Number { return _radius; }
		public function set radius(value:Number):void
		{
			if (value < 0) throw new RangeError("value must not greater than zero");
			
			_radius = value;
		}
		
		
		//-----------//
		//Constructor//
		//-----------//
		
		/**
		 * Creates a new <code>BasicCircle</code> object.
		 * 
		 * @param	x		The x coordinate of the center.
		 * @param	y		The y coordinate of the center.
		 * @param	radius	The radius of the circle.
		 */
		public function BasicCircle(x:Number=0, y:Number=0, radius:Number=0):void
		{
			this.x = x;
			this.y = y;
			this.radius = radius;
		}
		
		//--------------//
		//Public methods//
		//--------------//
		
		/**
		 * Retrieves if a specified point hits the <code>BasicCircle</code> or not.
		 * 
		 * @param	x			The x coordinate.
		 * @param	y			The y coordinate.
		 * @return	<code>true</code> if the point hit the <code>BasicCircle</code>.
		 */
		public function hitTestPoint(x:Number, y:Number):Boolean
		{
			var v:Vector = new Vector(x - this.x, y - this.y);
			return v.norm <= radius;
		}

		/**
		 * Creates a copy of the current <code>BasicCircle</code>.
		 * 
		 * @return	The copy of the current <code>BasicCircle</code>.
		 */
		public function clone():Object
		{
			return new BasicCircle(x, y, radius);
		}
		
		/**
		 * Retrieves the nearest <code>Point</code> that hits the <code>BasicCircle</code>.
		 * 
		 * @param	x	An arbitrary x coordinate.
		 * @param	y	An arbitrary y coordinate.
		 * @return	The nearest <code>Point</code> that hits the <code>BasicCircle</code>.
		 */
		public function getNearestHitPoint(x:Number, y:Number):Point
		{
			if (hitTestPoint(x, y)) return new Point(x, y);
			
			var s:Vector = new Vector(this.x, this.y);
			var v:Vector = new Vector(x - this.x, y - this.y);
			v.norm = radius;
			v.add(s);
			
			return v.toPoint();
		}
		
		/**
		 * Draws the current <code>BasicCircle</code> to the specified <code>Graphics</code> object.
		 * 
		 * @param	gr	The <code>Graphics</code> object to draw to.
		 */
		public function draw(gr:Graphics):void
		{
			gr.drawCircle(x, y, radius);
		}
	}
}
