package ch.capi.geom 
{
	import ch.capi.display.IDrawable;	
	import ch.capi.utils.ICloneable;	
	import ch.capi.display.IDragDelimiter;	
	
	import flash.geom.Rectangle;	
	import flash.display.Graphics;	
	import flash.geom.Point;
	
	/**
	 * Represents a rectangle.
	 * 
	 * @author		Cedric Tabin - thecaptain
	 * @version 	1.0
	 */
	public class BasicRectangle implements IDragDelimiter, ICloneable, IDrawable
	{
		//---------//
		//Variables//
		//---------//
		
		/**
		 * Defines the x coordinate.
		 */
		public var x:Number;
		
		/**
		 * Defines the y coordinate.
		 */
		public var y:Number;
		
		/**
		 * Defines the width.
		 */
		public var width:Number;
		
		/**
		 * Defines the height.
		 */
		public var height:Number;
		
		//-----------//
		//Constructor//
		//-----------//
		
		/**
		 * Creates a new <code>BasicRectangle</code> object.
		 * 
		 * @param	x		The x coordinate.
		 * @param	y		The y coordinate.
		 * @param	width	The width.
		 * @param	height	The height.
		 */
		public function BasicRectangle(x:Number=0, y:Number=0, width:Number=0, height:Number=0):void
		{
			this.x = x;
			this.y = y;
			this.width = width;
			this.height = height;
		}
		
		//--------------//
		//Public methods//
		//--------------//
		
		/**
		 * Retrieves if a specified point hits the <code>BasicRectangle</code> or not.
		 * 
		 * @param	x			The x coordinate.
		 * @param	y			The y coordinate.
		 * @return	<code>true</code> if the point hit the <code>BasicRectangle</code>.
		 */
		public function hitTestPoint(x:Number, y:Number):Boolean
		{
			return (x >= this.x && x <= this.x+width && y >= this.y && y <= this.y+height);
		}
		
		/**
		 * Retrieves the nearest <code>Point</code> that hits the <code>BasicRectangle</code>.
		 * 
		 * @param	x	An arbitrary x coordinate.
		 * @param	y	An arbitrary y coordinate.
		 * @return	The nearest <code>Point</code> that hits the <code>BasicRectangle</code>.
		 */
		public function getNearestHitPoint(x:Number, y:Number):Point
		{
			if (hitTestPoint(x, y)) return new Point(x, y);
			
			if (x < this.x && y < this. y) return new Point(this.x, this.y);
			if (x > this.x+width && y > this.y+height) return new Point(this.x+width, this.y+height);
			if (x < this.x && y > this.y+height) return new Point(this.x, this.y+height);
			if (x > this.x+width && y < this.y) return new Point(this.x+width, this.y);
			if (x < this.x) return new Point(this.x, y);
			if (x > this.x+width) return new Point(this.x+width, y);
			if (y < this.y) return new Point(x, this.y);
			if (y > this.y+height) return new Point(x, this.y+height);
			
			//this case should never append !
			throw new Error();
		}
		
		/**
		 * Draws the current <code>BasicRectangle</code> to the specified <code>Graphics</code> object.
		 * 
		 * @param	gr	The <code>Graphics</code> object to draw to.
		 */
		public function draw(gr:Graphics):void
		{
			gr.drawRect(x, y, width, height);
		}
		
		/**
		 * Creates a copy of the current object.
		 * 
		 * @return	The copy of the current object.
		 */
		public function clone():Object
		{
			return new BasicRectangle(x, y, width, height);
		}
		
		/**
		 * Retrieves a <code>flash.geom.Rectangle</code> from the current <code>BasicRectangle</code>.
		 * 
		 * @return	A <code>flash.geom.Rectangle</code>.
		 */
		public function toRectangle():Rectangle
		{
			return new Rectangle(x, y, width, height);
		}
	}
}
